import kinto.core
import webtest

from kinto.core.utils import random_bytes_hex
from pyramid.config import Configurator


def get_request_class(prefix):
    class PrefixedRequestClass(webtest.app.TestRequest):
        @classmethod
        def blank(cls, path, *args, **kwargs):
            path = "/%s%s" % (prefix, path)
            return webtest.app.TestRequest.blank(path, *args, **kwargs)

    return PrefixedRequestClass


class BaseWebTest(object):
    """Base Web Test to test your cornice service.

    It setups the database before each test and delete it after.
    """

    api_prefix = "v1"

    def __init__(self, *args, **kwargs):
        super(BaseWebTest, self).__init__(*args, **kwargs)
        self.app = self._get_test_app()
        self.headers = {"Content-Type": "application/json"}

    def _get_test_app(self, settings=None):
        config = self._get_app_config(settings)
        wsgi_app = config.make_wsgi_app()
        app = webtest.TestApp(wsgi_app)
        app.RequestClass = get_request_class(self.api_prefix)
        return app

    def _get_app_config(self, settings=None):
        config = Configurator(settings=self.get_app_settings(settings))
        kinto.core.initialize(config, version="1.0.1")
        return config

    def get_app_settings(self, additional_settings=None):
        """
        kinto.includes = kinto_pusher

        kinto.event_listeners = pusher
        kinto.event_listeners.pusher.use = kinto_pusher.listener
        kinto.event_listeners.pusher.resources = <list of resource names>
        kinto.event_listeners.pusher.channel = <channel-name or pattern>

        pusher.app_id = <pusher-app-id>
        pusher.key = <pusher-key>
        pusher.secret = <pusher-secret>
        """
        settings = kinto.core.DEFAULT_SETTINGS.copy()
        settings["includes"] = "kinto_pusher"
        settings["cache_backend"] = "kinto.core.cache.memory"
        settings["cache_backend"] = "kinto.core.cache.memory"
        settings["userid_hmac_secret"] = random_bytes_hex(16)
        settings["event_listeners"] = "pusher"
        settings["event_listeners.pusher.use"] = "kinto_pusher.listener"
        settings["event_listeners.pusher.resources"] = "records"
        pattern = "{bucket_id}-{collection_id}-{resource_name}"
        settings["event_listeners.pusher.channel"] = pattern
        settings["pusher.app_id"] = "12345"
        settings["pusher.key"] = "demo-key"
        settings["pusher.secret"] = "demo-secret"

        if additional_settings is not None:
            settings.update(additional_settings)
        return settings



import re

from kinto.core.listeners import ListenerBase


class Listener(ListenerBase):
    def __init__(self, channel, *args, **kwargs):
        super(Listener, self).__init__(*args, **kwargs)
        self.channel = channel

    def __call__(self, event):
        try:
            channel = self.channel.format(**event.payload)
        except KeyError:
            # In case we cannot build the channel name with the
            # current event just ignore it. Refs #19
            return

        channel = re.sub('[^a-zA-Z0-9_\\-]', '', channel)
        action = event.payload['action']

        payload = event.impacted_records

        registry = event.request.registry
        try:
            registry.pusher.trigger(channel, action, payload)
        except ValueError:
            # In case pusher refuse to send large payload try without
            # payload.
            payload = {"payload": "Too large to be sent from there."}
            registry.pusher.trigger(channel, action, payload)


def load_from_config(config, prefix=''):
    settings = config.get_settings()

    # XXX: this will only work if configured listener is called `pusher`
    channel = settings.get('event_listeners.pusher.channel',
                           '{bucket_id}-{collection_id}-{resource_name}')

    return Listener(channel)



import pkg_resources

from pusher import Pusher

#: Module version, as defined in PEP-0396.
__version__ = pkg_resources.get_distribution(__package__).version


def includeme(config):
    settings = config.get_settings()

    app_id = str(settings['pusher.app_id'])
    key = settings['pusher.key']
    secret = settings['pusher.secret']
    cluster = settings.get('pusher.cluster')

    config.registry.pusher = Pusher(app_id, key, secret, cluster=cluster)

    config.add_api_capability(
        "pusher",
        version=__version__,
        description="Notify Pusher when something changes.",
        url="https://github.com/Kinto/kinto-pusher",
        app_id=app_id,
        key=key)



import unittest
from . import BaseWebTest

from kinto_pusher import __version__ as pusher_version


class CapabilityTestView(BaseWebTest, unittest.TestCase):
    def test_pusher_capability(self):
        resp = self.app.get("/")
        capabilities = resp.json["capabilities"]
        self.assertIn("pusher", capabilities)
        expected = {
            "version": pusher_version,
            "url": "https://github.com/Kinto/kinto-pusher",
            "description": "Notify Pusher when something changes.",
            "app_id": "12345",
            "key": "demo-key",
        }
        self.assertEqual(expected, capabilities["pusher"])



#!/usr/bin/env python
import os
import codecs
from setuptools import setup

here = os.path.abspath(os.path.dirname(__file__))


def read_file(filename):
    """Open a related file and return its content."""
    with codecs.open(os.path.join(here, filename), encoding="utf-8") as f:
        content = f.read()
    return content


README = read_file("README.rst")
CHANGELOG = read_file("CHANGELOG.rst")

requirements = ["kinto", "pusher"]

test_requirements = [
    # TODO: put package test requirements here
]

setup(
    name="kinto-pusher",
    version="1.0.0",
    description="Plug Kinto notifications into Pusher.com",
    long_description=README + "\n\n" + CHANGELOG,
    author="Mathieu Leplatre",
    author_email="mathieu@leplat.re",
    url="https://github.com/Kinto/kinto-pusher",
    packages=["kinto_pusher"],
    package_dir={"kinto_pusher": "kinto_pusher"},
    include_package_data=True,
    install_requires=requirements,
    license="Apache License (2.0)",
    zip_safe=False,
    keywords="kinto pusher",
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: Apache Software License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
    ],
    test_suite="tests",
    tests_require=test_requirements,
)
